package com.example.chatbot;

import java.util.List;
import java.util.Map;

import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.example.chatbot.service.EmbedderClient;
import com.example.chatbot.service.WeaviateService;
import com.fasterxml.jackson.core.JsonProcessingException;

@RestController
@RequestMapping("/chat")
public class ChatController {

  private final EmbedderClient embedder;
  private final WeaviateService weaviate;
  private final OpenAiServiceWrapper openai;

  public ChatController(EmbedderClient embedder, WeaviateService weaviate, OpenAiServiceWrapper openai) {
    this.embedder = embedder;
    this.weaviate = weaviate;
    this.openai = openai;
  }

  @PostMapping("/ask")
  public ResponseEntity<String> ask(@RequestBody Map<String, String> payload) throws JsonProcessingException {
    String question = payload.get("question");
    List<Double> vector = embedder.embedText(question);
    List<String> context = weaviate.searchRelevant(question, vector);
    String answer = openai.processQuestion(question, context);
    return ResponseEntity.ok(answer);
  }

  @PostMapping("/ingest")
  public ResponseEntity<String> ingest(@RequestBody Map<String, String> payload) {
    String content = payload.get("content");
    List<Double> vector = embedder.embedText(content);
    weaviate.ingest(content, vector);
    return ResponseEntity.ok("Saved");
  }

  @PostMapping("/askSimple")
  public String ask(@RequestBody String question) {
    return openai.processQuestion2(question);
  }
}
