package com.example.chatbot.service;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;

@Service
public class WeaviateService {

  @Value("${weaviate.url}")
  private String weaviateUrl;

  private final RestTemplate restTemplate = new RestTemplate();

  public void ingest(String content, List<Double> vector) {
    Map<String, Object> data = Map.of("content", content);
    Map<String, Object> request = Map.of("class", "Document", "properties", data, "vector", vector);

    restTemplate.postForObject(weaviateUrl + "/v1/objects", request, String.class);
  }

  public List<String> searchRelevant(String question, List<Double> qVector) throws JsonProcessingException {
    HttpHeaders headers = new HttpHeaders();
    headers.setContentType(MediaType.APPLICATION_JSON);
    Map<String, Object> nearVector = Map.of("vector", qVector);
    String gql = """
        {
          Get {
            Document(nearVector: %s, limit: 3) {
              content
            }
          }
        }
        """.formatted(new ObjectMapper().writeValueAsString(nearVector));

    Map<String, String> query = Map.of("query", gql);
    HttpEntity<Map<String, String>> entity = new HttpEntity<>(query, headers);

    ResponseEntity<JsonNode> resp = restTemplate.postForEntity(weaviateUrl + "/v1/graphql", entity, JsonNode.class);

    List<String> results = new ArrayList<>();
    resp.getBody().at("/data/Get/Document").forEach(item -> results.add(item.get("content").asText()));
    return results;
  }
}
