package com.example.chatbot;

import static org.assertj.core.api.Assertions.assertThat;

import java.io.InputStream;
import java.util.List;
import java.util.Map;

import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.client.RestTemplate;
import org.yaml.snakeyaml.Yaml;

public class ChatControllerTest {

  private static final String API_URL = "http://localhost:8080/chat/askSimple";  // Change this if needed to match your server's address

  private static String embeddingUrl;

  @BeforeAll
  public static void loadYaml() {
    // This loads application.yml config. Can't use SpringBoot (magic) infraestruture because I can't setup the TestCase as @SpringBootTest
    Yaml yaml = new Yaml();
    try (InputStream in = ChatControllerTest.class.getClassLoader().getResourceAsStream("application.yml")) {
      Map<String, Object> obj = yaml.load(in);

      // Property embedding.url
      Map<String, Object> embedding = (Map<String, Object>) obj.get("embedding");
      embeddingUrl = (String) embedding.get("url");

    } catch (Exception e) {
      throw new RuntimeException("Failed to load YAML config", e);
    }
  }

  @Test
  public void testEmbeddingEndpoint() {
    String textToEmbedd = "ABC spent 5000 on gasoline in 2023";

    RestTemplate restTemplate = new RestTemplate();

    HttpHeaders headers = new HttpHeaders();
    headers.setContentType(MediaType.APPLICATION_JSON);

    Map<String, Object> request = Map.of("texts", List.of(textToEmbedd));
    HttpEntity<Map<String, Object>> entity = new HttpEntity<>(request, headers);

    Map response = restTemplate.postForObject(embeddingUrl, entity, Map.class);
    List<Double> embedding = (List<Double>) ((List<?>) response.get("embeddings")).get(0);

    System.out.println(embedding);
  }

  @Test
  public void testChatEndpoint() {
    String question = "How much did company ABC spend on Gasoline in 2023?";

    // Set the headers for the request
    HttpHeaders headers = new HttpHeaders();
    headers.setContentType(MediaType.TEXT_PLAIN);

    // Create the HTTP request entity with the question body and headers
    HttpEntity<String> request = new HttpEntity<>(question, headers);

    // Use RestTemplate to make the HTTP POST request
    RestTemplate restTemplate = new RestTemplate();
    ResponseEntity<String> response = restTemplate.exchange(API_URL, HttpMethod.POST, request, String.class);

    // Assert that the response status code is OK
    assertThat(response.getStatusCode()).isEqualTo(HttpStatus.OK);

    // Assert that the response body is not blank (contains a valid AI response)
    assertThat(response.getBody()).isNotBlank();
    System.out.println("AI Response: " + response.getBody());
  }

}
